package org.jutils.collection;

/**
 * Designed to handle m to n associations. if <code>bidirectional==true</code>, then asociations are stored both ways when added.
 * The elements stored for one key are stored in a Vector which is returned when asked for.
 * This Vector is passed as reference and may be modified at will.<br>
 * For an example, see the <code>main(String[])</code>-method, which is a testing method.
 * @see main(String[])
 * <p>------------------------------------------------------------------------------
 * <br>Name:                            Associator
 * <br>Project:                 jutils
 * <br>Copyright:               jutils.org, see license file.
 * <br>Creation date: (27.07.2000 12:47:27)
 * <br>Version:   <pre>$Id: Associator.java,v 1.10 2000/10/05 17:00:07 winkler Exp $</pre>
 * <br>------------------------------------------------------------------------------
 * <p>
 * @author: M.Winkler, doubleSlash GmbH
**/

import java.util.*;
public class Associator {
    /** Contains the keys and value vectors. */
    // private Hashtable relationalData_ = new Hashtable();
    private Map relationalData_ = new Hashtable();

    /** Defines wether or not this asociator works bidirectional. */
    private boolean bidirectional_ = false;

    /** Determines the behavior when an index is out of bounds, or if an element is not found. */
    private int indexOutOfBoundsBehavior_ = DynamicList.RETURN_NULL;
    private String listOpenBracket_ = "{";
    private String listCloseBracket_ = "}";
    private String listSeparator_ = ",";



    /**
       * OneToManyRelation constructor comment.
       */
    public Associator() {
        super();
    }



    /**
       * OneToManyRelation constructor comment.
       */
    public Associator(boolean bidirectional) {
        super();
        bidirectional_ = bidirectional;
    }



    /**
    * Defines a new association between this two values.
    * <br>Creation date: (27.07.2000 12:51:05)<br>
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @param key java.lang.Object
    * @param newValue java.lang.Object
    */
    public void associate(Object key, Object newValue) {
        pAssociate(key, newValue);
        if (bidirectional_)
            pAssociate(newValue, key);
    }



    /**
    * clear method comment.
    */
    public void clear() {
        relationalData_.clear();
    }



    /**
       * Code to perform when this object is garbage collected.
       *
       * Any exception thrown by a finalize method causes the finalization to
       * halt. But otherwise, it is ignored.
       */
    protected void finalize() throws Throwable {
        // Insert code to finalize the receiver here.
        // This implementation simply forwards the message to super.  You may replace or supplement this.
        super.finalize();
    }



    /**
    * getIndexOutOfBoundsBehavior method comment.
    */
    public int getIndexOutOfBoundsBehavior() {
        return indexOutOfBoundsBehavior_;
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.Object[]
     */
    public Object[] getKeysArray() {
        Object[] returnValue = null;
        List keysList = keysList();
        returnValue = keysList.toArray();
        return returnValue;
    }



    /**
       * Tests the class.
       * @param args an array of command-line arguments
       */
    public static void main(java.lang.String[] args) {
        System.out.println("Testing Associator ...");
        int errorCount = 0;
        System.out.println("initializing ...");
        Associator a = new Associator(false);
        int initialCount = 1000;
        long startTime = System.currentTimeMillis();
        for (int index = 0; index < initialCount; index++) {
            Integer key = new Integer(index);
            Integer value = new Integer(index + initialCount);
    
            // associationg unidirectional key with value
            a.associate(key, value);
    
            // associationg unidirectional key with itself
            a.associate(key, key);
        }
        long timeUsed = System.currentTimeMillis() - startTime;
        System.out.println("Needed " + timeUsed + " ms to create " + initialCount * 2 + " entries.");
        double timePerOp = (double) timeUsed / (initialCount * 2);
        System.out.println("That is " + timePerOp + " ms per operation.");
        System.out.println();
        System.out.println("Checking consistency ...");
        startTime = System.currentTimeMillis();
        for (int index = 0; index < initialCount; index++) {
            Integer key = new Integer(index);
            Integer value = new Integer(index + initialCount);
    
            // retrieving associations
            List associations = a.getAssociations(key);
            Integer a_1 = (Integer) associations.get(0);
            Integer a_2 = (Integer) associations.get(1);
            if (!value.equals(a_1)) {
                System.err.println("Error! association not found!");
                errorCount++;
            }
            if (!key.equals(a_2)) {
                System.err.println("Error! association not found!");
                errorCount++;
            }
    
            // check removal
            a.remove(key, value);
            associations = a.getAssociations(key);
            if (associations.size() > 1) {
                System.err.println("Error! count does not match aftrer removing assotiation for " + key + "!");
                System.err.println("Associations : " + associations.toString());
                errorCount++;
            }
        }
        timeUsed = System.currentTimeMillis() - startTime;
        System.out.println("Needed " + timeUsed + " ms to check " + initialCount + " entries.");
        System.out.println();
        System.out.println(a);
        System.out.println();
        System.out.println("" + errorCount + " errors found.");
    }



    /**
       * Does the association, one-way.
       * <br>Creation date: (27.07.2000 13:06:00)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @param key java.lang.Object
       * @param value java.lang.Object
       */
    private void pAssociate(Object key, Object value) {
    
        // retrieve existing vector
        Vector v = null;
        if (relationalData_.containsKey(key))
            v = (Vector) relationalData_.get(key);
        else {
            v = new Vector();
            relationalData_.put(key, v);
        }
        if (!v.contains(value)) {
            v.add(value);
        }
    }



    /**
       * Removes the key unidirectional.
       * <br>Creation date: (27.07.2000 12:56:41)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return boolean
       * @param key java.lang.Object
       * @param value java.lang.Object
       */
    private boolean pRemove(Object key, Object value) {
        boolean returnValue = false;
    
        // First, find key
        if (relationalData_.containsKey(key)) {
    
            // retrieve vector
            Vector v = (Vector) relationalData_.get(key);
    
            // is it present?!?
            if (v.contains(value)) {
    
                // return
                returnValue = true;
                v.remove(value);
            }
        }
        return returnValue;
    }



    /**
       * <br>Creation date: (27.07.2000 12:56:41)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return boolean
       * @param key java.lang.Object
       * @param value java.lang.Object
       */
    public boolean remove(Object key, Object value) {
        boolean returnValue = false;
        returnValue = pRemove(key, value);
        if (bidirectional_)
            returnValue = returnValue || pRemove(value, key);
        return returnValue;
    }



    /**
    * setListChars method comment.
    */
    public void setListChars(java.lang.String openBracket, java.lang.String separator, java.lang.String closeBracket) {
        listOpenBracket_ = openBracket;
        listSeparator_ = separator;
        listCloseBracket_ = closeBracket;
    }



    /**
       * Returns a String that represents the value of this object.
       * @return a string representation of the receiver
       */
    public String toString() {
        return toString(listOpenBracket_, listSeparator_, listCloseBracket_);
    }



    /**
    * toString method comment.
    */
    public java.lang.String toString(java.lang.String openBracket, java.lang.String separator, java.lang.String closeBracket) {
        StringBuffer returnValue = new StringBuffer();
        if (openBracket != null)
            returnValue.append(openBracket);
        Iterator keys = relationalData_.keySet().iterator();
        boolean alreadyAddedSomething = false;
        while (keys.hasNext()) {
            if (alreadyAddedSomething)
                returnValue.append(separator);
            returnValue.append('(');
            Object key = keys.next();
            returnValue.append(key);
            if (bidirectional_)
                returnValue.append('<');
            returnValue.append("=>");
            Object associations = relationalData_.get(key);
            returnValue.append(associations);
            returnValue.append(')');
            alreadyAddedSomething = true;
        }
        if (closeBracket != null)
            returnValue.append(closeBracket);
        return returnValue.toString();
    }



    /**
       * <br>Creation date: (27.07.2000 13:13:11)<br>
       * @author: M.Winkler, doubleSlash GmbH
       *
       * @return java.util.Vector
       * @param key java.lang.Object
       */
    public List getAssociations(Object key) {
        List returnValue = null;
        returnValue = (List) relationalData_.get(key);
        return returnValue;
    }



    /**
     * @author Michael Winkler, doubleSlash GmbH
     *
     * @return java.lang.Object[]
     */
    public List keysList() {
        List returnValue = null;
        // Enumeration keysEnumeration = relationalData_.keys();
        Set keysSet = relationalData_.keySet();
        returnValue = new Vector(keysSet);
        return returnValue;
    }
}

