package org.jutils.debugging;

import java.io.*;
/**
 * Provides an easy-to-usde way to debug source code.
 * Creation date: (03.11.2000 13:44:12)
 * @author: Michael Winkler
 */
public final class DebugTool {

    private static PrintWriter out_= new java.io.PrintWriter(System.out);

    public static boolean debugEnabled_= true;



    /**
     * Checks if the passed assertion is true. If not, then a new AssertionException is thrown with the new message.
     * Creation date: (03.11.2000 13:50:38)
     * @param assertionExpression boolean Insert here a boolean expression to define wether or not an assertion is met.
     * @param violationMessage java.lang.String A message that is returned when an asserton is NOT met. Be detailed!
     * @exception org.jutils.debugging.AssertionException Thrown when the passed expression is false.
     * @see org.jutils.debugging.AssertionException 
     */
    public static void checkAssertion(
        boolean assertionExpression, 
        String violationMessage)
        throws AssertionException {
        if (!assertionExpression)
            throw new AssertionException(violationMessage);
    }



    /**
     * Insert the method's description here.
     * Creation date: (07.11.2000 18:00:25)
     * @return java.io.PrintWriter
     */
    public static java.io.PrintWriter getOut() {
        return out_;
    }



    /**
     * Insert the method's description here.
     * Creation date: (07.11.2000 17:41:32)
     * @return boolean
     */
    public static boolean isDebugEnabled() {
        return debugEnabled_;
    }



    /**
     * Does a plain log output to the console.
     * Creation date: (03.11.2000 14:11:55)
     * @param sender java.lang.String
     * @param message java.lang.String
     */
    public static void logDebug(String sender, String message) {
        if (sender == null)
            sender= "<unknown>";
        if (message == null)
            message= "<ping>";
        out_.println(sender + '\t' + message);
        out_.flush();
    }



    /**
     * Insert the method's description here.
     * Creation date: (07.11.2000 17:41:32)
     * @param newDebugEnabled boolean
     */
    public static void setDebugEnabled(boolean newDebugEnabled) {
        debugEnabled_= newDebugEnabled;
    }



    /**
     * Insert the method's description here.
     * Creation date: (07.11.2000 18:00:25)
     * @param newOut java.io.PrintWriter
     */
    public static void setOut(java.io.PrintWriter newOut) {
        out_= newOut;
    }
}

